/**
 * @file csocp_solver.h
 * @brief 求解(solver)模块
 * This file implements solver module.
 * @version 0.1
 * @date 2021-09-17
 * 
 * @copyright Copyright (c) 2021
 * 
 */

#ifndef __CSOCP_SOLVER_H__
#define __CSOCP_SOLVER_H__

#include "csocp_config.h"

/** 计算初始点 Compute the starting point */
c_int c_solver_startpoint(csocp_prob *prob);

/** 预测步 Predicator step */
void c_solver_predictor(csocp_prob *prob, c_real *p_res);

/** 中心化步 Centering step */
void c_solver_centering(csocp_prob *prob);

/** 修正步 Corrector step */
void c_solver_corrector(csocp_prob *prob, c_real res);

/** 获取CSOCP求解器求解问题的内存需求量 Get memory requirement of CSOCP solver */
c_int c_solver_getmem(c_int n,    /**< 变量数量 Number of variables */
                      c_int m,    /**< 不等式约束数量 Number of inequalities */
                      c_int p,    /**< 等式约束数量 Number of equalities */
                      c_int nnzA, /**< 等式约束矩阵A的非零元个数 Number of nonzeros of A */
                      c_int nnzG  /**< 不等式约束矩阵G的非零元个数 Number of nonzeros of G */
);

#endif
