/**
 * @file struct_attr.h
 * @brief 定义属性(attribute)结构体. 本模块用来提高工程代码质量。
 * This is the definition of attribute structure. For better code engineering.
 * @version 0.1
 * @date 2021-09-17
 * 
 * @copyright Copyright (c) 2021
 * 
 */

#ifndef __STRUCT_ATTR_H__
#define __STRUCT_ATTR_H__

#include "csocp_config.h"

/** 属性结构体定义 Definition of attributes */
struct c_attr_s {
  c_int status;  /**< 求解状态 Solving status */
  c_int ispinf;  /**< 是否原始问题不可行 Is primal infeasible */
  c_int isdinf;  /**< 是否对偶问题不可行 Is dual infeasible */
  c_int itercnt; /**< 迭代次数计数 Iteration count */

  c_real loadtime;  /**< 加载时间 Loading time */
  c_real solvetime; /**< 求解时间 Solving time */

  c_real pobj; /**< 原始问题目标函数值 Primal objective value */
  c_real dobj; /**< 对偶问题目标函数值 Dual objective value */

  c_real pres;    /**< 原始问题残差 Primal residual */
  c_real dres;    /**< 对偶问题残差 Dual residual */
  c_real pinfres; /**< 原始问题不可行性残差 Primal residual of infeasibilities */
  c_real dinfres; /**< 对偶问题不可行性残差 Dual residual of infeasiblities */

  c_real absgap; /**< 绝对间隙 Absolute gap */
  c_real relgap; /**< 相对间隙 Relative gap */

  c_real sigma;
  c_real mu;

  c_real kap_t;
  c_real kap; /** kappa HSD */
  c_real tau; /** tau HSD */
  c_real kapaff;
  c_real tauaff;

  c_real step;
  c_real step_aff;

  c_real nx;
  c_real ny;
  c_real nz;
  c_real ns;

  c_real resx0;
  c_real resy0;
  c_real resz0;

  c_real cx;
  c_real by;
  c_real hz;
};

#endif
