/**
 * @file struct_param.h
 * @brief 定义参数(parameter)结构体。本模块用来提高工程代码质量。
 * This is the definition of parameter structure. For better code engineering.
 * @version 0.1
 * @date 2021-09-17
 * 
 * @copyright Copyright (c) 2021
 * 
 */

#ifndef __STRUCT_PARAM_H__
#define __STRUCT_PARAM_H__

#include "csocp_config.h"

/** 参数结构体定义 Definition of parameters */
struct c_param_s {
  c_real feastol; /**< 原始问题/对偶问题的不可行容限 Primal/dual infeasibility tolerance */
  c_real abstol;  /**< 对偶间隙的绝对容限 Absolute tolerance on duality gap */
  c_real reltol;  /**< 对偶间隙的相对容限 Relative tolerance on duality gap */
  
  c_int iriter;     /**< Number of IR steps */
  c_int irfact;     /**< Factor of IR error */
  c_real linsystol; /**< 线性系统容差 Linear system tolerance */

  c_real deltastat; /**< 静态正则化参数 Static regularization parameter */
  c_real delta;     /**< 动态正则化参数 Dynamic regularization parameter */
  c_real epsilon;   /**< epsilon容差 Epsilon tolerance */

  c_int scaleiter; /**< 缩放功能迭代次数限制 Iteration limit of scaling method */
  c_real scaletol; /**< 缩放容差 Scaling tolerance */

  c_real gamma; /**< 步长缩放系数 Scaling factor of step length */

  c_int maxit;    /**< 最大迭代次数 Maximum number of iterations */

  c_real minstep; /**< 允许的最小步长 Smallest step allowed */
  c_real maxstep; /**< 允许的最大步长 Largest step allowed */

  c_real minsigma; /**< 始终进行中心化的参数 Always do some centering */
  c_real maxsigma; /**< 始终不完全中心化的参数 Never fully centering */
};

#endif
