/**
 * This file implements Sparse cholesky factorization module in CSOCP solver
 */

#include "csocp_config.h"
#ifdef CSOCP_WITH_DEBUG
#include <stdio.h>
#endif

/* Symbolic factorization */
void c_ldl_symbolic(c_int ndim,   /* Dimension of KKT matrix */
                    c_int *Ajc,   /* Input begin pointer of KKT matrix */
                    c_int *Air,   /* Input index pointer of KKT matrix */
                    c_int *Ljc,   /* Output begin pointer of matrix */
                    c_int *etree, /* Output elimination tree */
                    c_int *lnz,   /* Temporary non-zeros count */
#ifndef CSOCP_WITH_DEBUG
                    c_int *flag /* Temporary visit flag */
#else
                    c_int *flag,  /* Temporary visit flag */
                    FILE *dbgfile /* Debug log file */
#endif
);

/* Numerical factorization */
void c_ldl_numeric(c_int ndim,     /* Dimension of KKT matrix */
                   c_int *Ajc,     /* Input begin pointer of KKT matrix */
                   c_int *Air,     /* Input index pointer of KKT matrix */
                   c_real *Apr,    /* Input element pointer of KKT matrix */
                   c_int *Ljc,     /* Input begin pointer of L matrix */
                   c_int *etree,   /* Input elimination tree */
                   c_int *sign,    /* Input sign vector */
                   c_real eps,     /* Input regularization threshold */
                   c_real delta,   /* Input regularization parameter */
                   c_int *Lir,     /* Output index pointer of L matrix */
                   c_real *Lpr,    /* Output element pointer of L matrix */
                   c_real *D,      /* Output diagonal D matrix */
                   c_real *col,    /* Temporary column vector */
                   c_int *lnz,     /* Temporary non-zeros count of L matrix */
                   c_int *pattern, /* Temporary non-zeros pattern */
#ifndef CSOCP_WITH_DEBUG
                   c_int *flag /* Temporary flag for visit status */
#else
                   c_int *flag,   /* Temporary flag for visit status */
                   FILE *dbgfile  /* Debug log file */
#endif
);

/* Solve L*x = b */
void c_ldl_lsolve(c_int ndim,  /* Dimension of L matrix */
                  c_real *b,   /* Input RHS value */
                  c_int *Ljc,  /* Input begin pointer of L matrix */
                  c_int *Lir,  /* Inut index pointer of L matrix */
                  c_real *Lpr, /* Input element pointer of L matrix */
#ifndef CSOCP_WITH_DEBUG
                  c_real *x /* Output solution value */
#else
                  c_real *x,      /* Output solution value */
                  FILE *dbgfile   /* Debug log file */
#endif
);

/* Solve D*x = b */
void c_ldl_dsolve(c_int ndim, /* Dimension of diagonal matrix */
                  c_real *x,  /* Input + Output value */
#ifndef CSOCP_WITH_DEBUG
                  c_real *D /* Input diagonal matrix */
#else
                  c_real *D,      /* Input diagonal matrix */
                  FILE *dbgfile   /* Debug log file */
#endif
);

/* Solve L'*x = b */
void c_ldl_ltsolve(c_int ndim, /* Dimension of L matrix */
                   c_real *x,  /* Input + Output value */
                   c_int *Ljc, /* Input begin pointer of L matrix */
                   c_int *Lir, /* Inut index pointer of L matrix */
#ifndef CSOCP_WITH_DEBUG
                   c_real *Lpr /* Input element pointer of L matrix */
#else
                   c_real *Lpr,   /* Input element pointer of L matrix */
                   FILE *dbgfile  /* Debug log file */
#endif
);
