/**
 * This is the definition of attribute structure in CSOCP solver
 *
 * For better code engineering.
 */

#ifndef __STRUCT_ATTR_H__
#define __STRUCT_ATTR_H__

#include "csocp_config.h"

/* Definition of attributes */
struct c_attr_s {
  /* Public attributes */
  c_int status;  /* Solving status */
  c_int ispinf;  /* Is primal infeasible */
  c_int isdinf;  /* Is dual infeasible */
  c_int itercnt; /* Iteration count */

  c_real pobj;    /* Primal objective value */
  c_real dobj;    /* Dual objective value */
  c_real pres;    /* Primal residual */
  c_real dres;    /* Dual residual */
  c_real pinfres; /* Primal residual of infeasibilities */
  c_real dinfres; /* Dual residual of infeasiblities */
  c_real absgap;  /* Absolute gap */
  c_real relgap;  /* Relative gap */

  /* Statistics */
  c_int n;        /* Number of variables */
  c_int m;        /* Number of inequalities */
  c_int p;        /* Number of equalities */
  c_int l;        /* Number of LP cones */
  c_int ncones;   /* Number of SOC cones */
  c_int nconedim; /* Sum of SOC dimensions */
  c_int nnzA;     /* Number of nonzeros in A */
  c_int nnzG;     /* Number of nonzeros in G */
  c_int nK;       /* Number of columns in KKT */
  c_int nnzK;     /* Number of nonzeros in KKT */
  c_int nnzL;     /* Number of nonzeros in factor L */
  c_int nTmpI;    /* Length of temporary integer workspace */
  c_int nTmpD;    /* Length of temporary double workspace */

  c_real load_time;  /* Loading time */
  c_real solve_time; /* Solving time */
#ifdef CSOCP_WITH_STATS
  c_int total_mem;  /* Total memory */
  c_int cone_mem;   /* Cone memory */
  c_int kkt_mem;    /* KKT memory */
  c_int solver_mem; /* Solver memory */
  c_int order_mem;  /* Ordering memory */
  c_int tmp_mem;    /* Temporary memory */
  c_int free_mem;   /* Alloc-free memory */
#endif

  /* Private attributes */
  c_real sigma; /* sigma */
  c_real mu;    /* mu */

  c_real kap_t;    /* kappa / tau */
  c_real kap;      /* kappa */
  c_real tau;      /* tau */
  c_real kapaff;   /* Affine kappa */
  c_real tauaff;   /* Affine tau */
  c_real step;     /* Step length */
  c_real step_aff; /* Affine step length */

  c_real nx; /* Norm of x */
  c_real ny; /* Norm of y */
  c_real nz; /* Norm of z */
  c_real ns; /* Norm of s */

  c_real cx; /* c'*x */
  c_real by; /* b'*y */
  c_real hz; /* h'*z */
};

#endif
