/**
 * This is definition of cone module for the CSOCP solver
 */

#ifndef __STRUCT_CONE_H__
#define __STRUCT_CONE_H__

#include "csocp_config.h"
#ifdef CSOCP_WITH_DEBUG
#include <stdio.h>
#endif

/* Structure for LP cone */
struct c_lp_s {
  c_int ndim;    /* Dimension of LP cone */
  c_int *idxmap; /* Map between v and element index in KKT matrix */
  c_real *w;     /* Scaling of LP cone */
  c_real *v;     /* Squared scaling of LP cone: w^2 */
};

/* Structure for SOC cone */
struct c_soc_s {
  c_int ndim;    /* Dimension of SOC cone */
  c_int *idxmap; /* Map between diagonal D and element index in KKT matrix */
  c_real *q;     /* wbar */
  c_real eta;    /* (sres / zres)^(0.25) */
  c_real eta_q;  /* (sres / zres)^(0.5) */
  c_real d1;     /* D[0] */
  c_real u0;     /* eta */
  c_real u1;     /* u = [u0; u1 * w] */
  c_real v1;     /* v = [0; v1 * w] */
};

/* The main cone structure */
struct c_cone_s {
#ifdef CSOCP_WITH_DEBUG
  FILE *dbgfile; /* Debug log file */
#endif

  c_lp *lp; /* LP cone */

  c_int nsoc;  /* Number of SOC cones */
  c_soc **soc; /* SOC cones */
};

#endif
