/**
 * This is the definition of parameter structure in CSOCP solver
 *
 * For better code engineering.
 */

#ifndef __STRUCT_PARAM_H__
#define __STRUCT_PARAM_H__

#include "csocp_config.h"

/* Definition of parameters */
struct c_param_s {
  c_int maxit; /* Maximum number of iterations */

  c_int rlxfactor;    /* Maximum factor for relax precision */
  c_real feastol;     /* Primal/dual feasibility tolerance */
  c_real abstol;      /* Absolute tolerance */
  c_real reltol;      /* Relative tolerance */
  c_real feastol_rlx; /* Relaxed primal/dual feasibility tolerance */
  c_real abstol_rlx;  /* Relaxed absolute tolerance */
  c_real reltol_rlx;  /* Relaxed relative tolerance */

  c_int scalemethod; /* Scaling method: (-1: Auto, 0: Ruiz, 1: Pock-Chambolle) */
  c_int scaleiter;   /* Iteration limit of ruiz scaling method */
  c_real scaletol;   /* Scaling tolerance */

  c_int mustrategy; /* Mu update strategy (0: Default, 1: LOQO) */
  c_real alphaloqo; /* Alpha value of LOQO strategy */

  c_int iriter;     /* Number of IR steps */
  c_int irfact;     /* Factor of IR error */
  c_real linsystol; /* Linear system tolerance */
  c_real deltastat; /* Static regularization parameter */
  c_real delta;     /* Dynamic regularization parameter */
  c_real epsilon;   /* Epsilon tolerance */

  c_real gamma;    /* Scaling factor of step length */
  c_real minstep;  /* Smallest step allowed */
  c_real maxstep;  /* Largest step allowed */
  c_real minsigma; /* Always do some centering */
  c_real maxsigma; /* Never fully centering */
};

#endif
