/**
 * This is the definition of problem structure in CSOCP solver
 *
 * For better code engineering.
 */

#ifndef __STRUCT_PROB_H__
#define __STRUCT_PROB_H__

#include "csocp_config.h"
#ifdef CSOCP_WITH_DEBUG
#include <stdio.h>
#endif

/* Structrue of CSOCP problem */
struct c_prob_s {
#ifdef CSOCP_WITH_DEBUG
  FILE *dbgfile; /* Debug file */
#endif

  c_int n;          /* Number of variables */
  c_int m;          /* Number of inequalities */
  c_int p;          /* Number of equalities */
  c_int degree;     /* Degree of the cone */
  c_spmat *A;       /* Equality matrix */
  c_spmat *G;       /* Inequality matrix */
  c_real *c;        /* Objective costs */
  c_real *b;        /* RHS of equalities */
  c_real *h;        /* RHS of cones */
  c_cone *cone;     /* Cone */
  c_kkt *kkt;       /* KKT System */
  c_param *param;   /* Parameters */
  c_buffer *buffer; /* Integer and double buffer */

  c_real *facX; /* Scaling factors of variables */
  c_real *facA; /* Scaling factors of equality matrix */
  c_real *facG; /* Scaling factors of inequality matrix */

  c_real *rx; /* Residual of x */
  c_real *ry; /* Residual of y */
  c_real *rz; /* Residual of z */

  c_real *dx1; /* Search direction */
  c_real *dy1; /* Search direction */
  c_real *dz1; /* Search direction */

  c_real *dx2; /* Search direction */
  c_real *dy2; /* Search direction */
  c_real *dz2; /* Search direction */

  c_real *dsaff;   /**/
  c_real *dzaff;   /**/
  c_real *dsaff_w; /**/
  c_real *dzaff_w; /**/

  c_real *lambda; /* Scaled cone variables */

  c_real *x;    /* Primal variables */
  c_real *y;    /* Dual variables for equalities */
  c_real *z;    /* Dual variables for inequalities */
  c_real *s;    /* Primal slack variables for inequalities */
  c_attr *attr; /* Attributes */

  c_real *best_x;    /* Best primal variables */
  c_real *best_y;    /* Best dual variables for equalities */
  c_real *best_z;    /* Best variables for inequalities */
  c_real *best_s;    /* Best slack variables for inequalities */
  c_attr *best_attr; /* Best attributes */

  c_real resx0; /* Initial residual of x */
  c_real resy0; /* Initial residual of y */
  c_real resz0; /* Initial residual of z */
};

#endif
