/**
 * This file implements routines for buffer in CSOCP solver
 */

#include "csocp_config.h"

#include "tool_buffer.h"
#include "tool_math.h"
#include "tool_memory.h"

#include "struct_buffer.h"

#include <stdlib.h>

/* Create buffer */
void c_buffer_create(c_buffer **p_buffer, /* Pointer to buffer */
                     c_int isize,         /* Size of integer buffer */
                     c_int dsize          /* Size of double buffer */
) {
  c_buffer *buffer = NULL;

  CSOCP_INITZ(buffer, 1);
  CSOCP_INIT(buffer->ibuf, isize);
  CSOCP_INIT(buffer->dbuf, dsize);

  buffer->isize = isize;
  buffer->dsize = dsize;

  *p_buffer = buffer;

  return;
}

/* Free buffer */
void c_buffer_free(c_buffer **p_buffer /* Pointer to buffer */
) {
  if (p_buffer != NULL && *p_buffer != NULL) {
    CSOCP_FREE((*p_buffer)->ibuf);
    (*p_buffer)->isize = 0;
    (*p_buffer)->ibeg = 0;

    CSOCP_FREE((*p_buffer)->dbuf);
    (*p_buffer)->dsize = 0;
    (*p_buffer)->dbeg = 0;

    CSOCP_FREE(*p_buffer);
  }
  return;
}

/* Get memory usage of buffer */
c_int c_buffer_getmem(c_int isize, /* Size of integer buffer */
                      c_int dsize  /* Size of double buffer */
) {
  c_int buffsize = 0;

  /* Memory of buffer structure */
  buffsize += sizeof(c_buffer);

  /* Memory of buffer data */
  buffsize += isize * sizeof(c_int); /* ibuf */
  buffsize += sizeof(c_int);         /* isize */
  buffsize += sizeof(c_int);         /* ibeg */

  buffsize += dsize * sizeof(c_real); /* dbuf */
  buffsize += sizeof(c_int);          /* dsize */
  buffsize += sizeof(c_int);          /* dbeg */

  return buffsize;
}
