/**
 * This file implements interrupt handling for the CSOCP solver.
 *
 * Notice that this module is needed only during development, and should be disabled
 * when used in production.
 *
 * This module serves as utility module, don't waste time in reading it.
 *
 * To enable this module, just define macro 'CSOCP_WITH_INTERRUPT' when compiling it;
 * othewise, it is disabled.
 */

#include "csocp_config.h"

#include "tool_interrupt.h"

#if defined(MATLAB_MEX_FILE) // Used only in MATLAB
#include "mex.h"

/* These are undocumented MATLAB functions dealing with interrupt action */
extern bool utIsInterruptPending(void);
extern bool utSetInterruptEnabled(bool);

static int is_interrupted;

/* Create interrupt handler */
void c_interrupt_register(void) {
#ifdef CSOCP_WITH_INTERRUPT
  is_interrupted = (int)utSetInterruptEnabled(true);
#else
  return;
#endif
}
/* Remove interrupt handler */
void c_interrupt_reset(void) {
#ifdef CSOCP_WITH_INTERRUPT
  utSetInterruptEnabled((bool)is_interrupted);
#else
  return;
#endif
}
/* Check for interrupt action */
int c_interrupt_check(void) {
#ifdef CSOCP_WITH_INTERRUPT
  return (int)utIsInterruptPending();
#else
  return 0;
#endif
}
#else
#ifdef CSOCP_WITH_INTERRUPT
#include <signal.h>
#include <stdio.h>

static int is_interrupted;
#endif

static void interrupt_callback(int signum) {
#ifdef CSOCP_WITH_INTERRUPT
  is_interrupted = 1;
  printf("User interrupted.\n");
#else
  return;
#endif
}

/* Create interrupt handler */
void c_interrupt_register(void) {
#ifdef CSOCP_WITH_INTERRUPT
  is_interrupted = 0;
  signal(SIGINT, interrupt_callback);
#else
  return;
#endif
}
/* Remove interrupt handler */
void c_interrupt_reset(void) {
#ifdef CSOCP_WITH_INTERRUPT
  is_interrupted = 0;
  signal(SIGINT, SIG_DFL);
#else
  return;
#endif
}
/* Check for interrupt action */
int c_interrupt_check(void) {
#ifdef CSOCP_WITH_INTERRUPT
  return is_interrupted;
#else
  return 0;
#endif
}
#endif
