/**
 * This file implements routines for memory management in CSOCP solver
 */

#include "csocp_config.h"

#include "tool_memory.h"

#include <stdlib.h>
#include <string.h>

#ifdef MATLAB_MEX_FILE
#include "mex.h"
#define c_malloc mxMalloc
#define c_realloc mxRealloc
#define c_free mxFree
#else
#define c_malloc malloc
#define c_realloc realloc
#define c_free free
#endif

/* Allocate memory */
void c_mem_create(void **p_ptr, /* Pointer to input data */
                  c_int size,   /* Size of allocated memory */
                  c_int ifzero  /* Whether to initialize by zeros */
) {
  void *ptr = NULL;

  if (p_ptr == NULL) {
    return;
  }

  if (size > 0) {
    if ((ptr = c_malloc(size)) == NULL) {
      return;
    }
  }

  if (ifzero) {
    memset(ptr, 0, size);
  }

  *p_ptr = ptr;

  return;
}

/* Resize memory */
void c_mem_resize(void **p_ptr, /* Pointer to the input data */
                  c_int size    /* Size of allocated memory */
) {
  void *ptr = NULL;

  if (p_ptr == NULL) {
    return;
  }

  if (*p_ptr == NULL) {
    c_mem_create(p_ptr, size, 0);
    return;
  }

  if (size == 0) {
    c_mem_free(p_ptr);
    return;
  }

  if ((ptr = c_realloc(*p_ptr, size)) == NULL) {
    return;
  }
  *p_ptr = ptr;

  return;
}

/* Copy memory */
void c_mem_copy(void *dst, /* Pointer to the desination data */
                void *src, /* Pointer to the source data */
                c_int size /* Size of allocated memory */
) {
  memcpy(dst, src, size);
}

/* Free memory */
void c_mem_free(void **p_ptr /* Pointer to input data */
) {
  if (p_ptr != NULL && *p_ptr != NULL) {
    c_free(*p_ptr);
    *p_ptr = NULL;
  }
  return;
}
