/**
 * This file implements routines for memory management in CSOCP solver
 */

#ifndef __TOOL_MEMORY_H__
#define __TOOL_MEMORY_H__

#include "csocp_config.h"

/* Allocate memory */
void c_mem_create(void **p_ptr, /* Pointer to input data */
                  c_int size,   /* Size of allocated memory */
                  c_int ifzero  /* Whether to initialize by zeros */
);

/* Resize memory */
void c_mem_resize(void **p_ptr, /* Pointer to the input data */
                  c_int size    /* Size of allocated memory */
);

/* Copy memory */
void c_mem_copy(void *dst, /* Pointer to the desination data */
                void *src, /* Pointer to the source data */
                c_int size /* Size of allocated memory */
);

/* Free memory */
void c_mem_free(void **p_ptr /* Pointer to input data */
);

#define CSOCP_INIT(ptr, num) c_mem_create((void **)&(ptr), sizeof(*(ptr)) * (num), 0)
#define CSOCP_INITZ(ptr, num) c_mem_create((void **)&(ptr), sizeof(*(ptr)) * (num), 1)
#define CSOCP_RESIZE(ptr, num) c_mem_resize((void **)&(ptr), sizeof(*(ptr)) * (num))
#define CSOCP_COPY(dst, src, num)                                                      \
  c_mem_copy((void *)(dst), (void *)(src), sizeof(*(dst)) * (num))
#define CSOCP_FREE(ptr) c_mem_free((void **)&(ptr))

#endif
