/**
 * This file implements timer for the CSOCP solver.
 *
 * Notice that this module is needed only during development, and should be
 * disabled when used in production.
 *
 * This module serves as utility module, don't waste time in reading it.
 */

#include "csocp_config.h"

#include "tool_time.h"

#ifdef _WIN32
#ifdef CSOCP_WITH_TIMER
#include <windows.h>
#endif
c_real c_timer_elapsed() {
#ifdef CSOCP_WITH_TIMER
  SYSTEMTIME system_time;
  FILETIME file_time;
  c_real time = 0.0;

  GetSystemTime(&system_time);
  SystemTimeToFileTime(&system_time, &file_time);

  time = 1.0 * file_time.dwLowDateTime;
  time += 4294967296.0 * file_time.dwHighDateTime;
  time *= 1e-7;
  return time;
#else
  return 0.0;
#endif
}
#else
#ifdef CSOCP_WITH_TIMER
#ifdef CSOCP_WITH_WALLTIME
#include <sys/time.h>
#else
#include <time.h>
#endif
#include <stdlib.h>
#endif
c_real c_timer_elapsed() {
#ifdef CSOCP_WITH_TIMER
#ifdef CSOCP_WITH_WALLTIME
  c_real walltime;
  struct timeval tv;
  gettimeofday(&tv, NULL);
  walltime = (c_real)tv.tv_sec;
  walltime += (c_real)tv.tv_usec * 0.000001;
  return walltime;
#else
  return (c_real)clock() / CLOCKS_PER_SEC;
#endif
#else
  return 0.0;
#endif
}
#endif
